""" Migration to create necessary permissions for API functions.

Generated by Django 4.2.3 on 2023-08-17
"""

from django.db import migrations, models


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_linked_records_app", "0002_alter_pidxpath_xpath"),
    ]

    operations = [
        migrations.CreateModel(
            name="LinkedRecords",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_linked_records_app",
                "permissions": (
                    ("read_pid_settings", "Can read pid settings"),
                ),
                "default_permissions": (),
            },
        ),
    ]
