{% extends "core_main_app/_render/admin/theme/tools/section.html" %}
{% block section_title %}PID settings{% endblock %}

{% block section_content %}
	{% include 'core_linked_records_app/admin/pid_settings/box.html' %}
{% endblock %}